package ema.tpjee.banking.controlerv1;

import java.io.IOException;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import ema.tpjee.banking.dao.BankDao;
import ema.tpjee.banking.dao.BankDaoImpl;
import ema.tpjee.banking.model.Bank;
import ema.tpjee.utils.ServletUtils;

/**
 * Servlet implementation class for Servlet: BankingControler
 * 
 */
public class BankControler extends javax.servlet.http.HttpServlet implements
		javax.servlet.Servlet {
	static final long serialVersionUID = 1L;

	private BankDao bankDao;

	/**
	 * Constructeur Le DAO est stateless, il peut donc tre li  la servlet
	 * (pas de pb d'accs concurrent)
	 */
	public BankControler() {
		super();
		System.out.println("creating BankControler");
		bankDao = new BankDaoImpl();
	}

	/**
	 * reconstituer le nouvel objet  partir des paramtres de la request les
	 * frameworks le font par introspection
	 * 
	 * @param request
	 * @return
	 */
	private Bank formValue(HttpServletRequest request) {
		if (request.getParameter("id") == null)
			return null;
		Bank result = new Bank();
		try {
			int rid = Integer.parseInt(request.getParameter("id"));
			result.setId(rid);
		} catch (Exception e) {

		}
		result.setName(request.getParameter("name"));
		result.setAddress(request.getParameter("address"));
		result.setPhone(request.getParameter("phone"));
		try {
			result.setZipCode(request.getParameter("zipCode"));
		} catch (Exception e) {
		}
		return result;
	}

	/**
	 * modifier l'objet au sein du modle aprs validation
	 * 
	 * @param request
	 * @param b
	 * @return
	 */
	private Bank doModify(HttpServletRequest request, Bank b) {
		Bank result = null;
		try {
			//validate(b); //en attente
			result = bankDao.merge(b); // le modle est mis  jour, si pas
			// d'exception
		} catch (Exception e) {
			result = b; // formBean is returned but not modified in persistence
			// layer
			System.out.println("error " + e.getMessage());
			request.setAttribute("error", "erreur " + e.getMessage());
		}
		return result;
	}	
	
	
	/**
	 * Mthode recevant l'action en provenance du client On souhaiterait
	 * gnraliser le mcanisme Cette gnralisation est ralise par les
	 * frameworks (struts, spring, etc..) et par JSF
	 */
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {

		request.getSession().setAttribute("controler-class",
				this.getClass().getName());
		ServletUtils.debugRequestParameters(this.getClass(), request);
		ServletUtils.debugSessionAttributes(this.getClass(), request);

		String page = request.getParameter("page");
		String cmd = request.getParameter("cmd");

		System.out.println("cmd = " + cmd);

		String fw = "/";
		int id = 0;
		try {
			id = Integer.parseInt(request.getParameter("id"));
		} catch (Exception e) {

		}

		Bank bean = bankDao.find(id);
		
		if ("editer".equals(cmd) || cmd == null)
			fw = "/pages/" + page + ".jsp";
		else if ("suivant".equals(cmd)) {
			bean = bankDao.next(bean);
			fw = "/pages/" + page + ".jsp";
		} else if ("precedent".equals(cmd)) {
			bean = bankDao.prior(bean);
			fw = "/pages/" + page + ".jsp";
		} else if ("dupliquer".equals(cmd)) {
			bean = bankDao.clone(bean);
			fw = "/pages/" + page + ".jsp";
		} else if ("supprimer".equals(cmd)) {
			bean = bankDao.delete(bean);
			fw = "/pages/" + page + ".jsp";
		} else if ("modifier".equals(cmd)){
			bean = formValue(request);
			bean = doModify(request, bean);
			fw = "/pages/" + page + ".jsp";
		}
		
		else {
			fw = "/";
			System.out.println("commande inconnue");
		}

		request.getSession().setAttribute("currentbank", bean);
		request.getSession().setAttribute("banks", bankDao.getList());
		
		System.out.println("forwarding to: " + fw);
		getServletContext().getRequestDispatcher(fw).forward(request, response);
	}



	/*
	 * (non-Java-doc)
	 * 
	 * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest request,
	 *      HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		doGet(request, response);
	}

	@Override
	public void init() throws ServletException {
		bankDao.populate();
		super.init();
	}
}